@@ -7,17 +7,17 @@
  * distributed with this software for terms of use.
  */
 
-struct timer_list {
-        struct timer_list *next;
-        struct timer_list *prev;
+struct timer_lst {
+        struct timer_lst *next;
+        struct timer_lst *prev;
         unsigned long expires;			/* how long till expiration */
 	unsigned long expected;			/* expected time of timeout */
-        unsigned long data;			/* data to pass to function */
-        void (*function)(unsigned long);	/* func to call on timeout */
+        void *data;				/* data to pass to function */
+        void (*function)(void *);		/* func to call on timeout */
 };
 
-extern void add_timer(struct timer_list * timer);
-extern int  del_timer(struct timer_list * timer);
-extern void init_timer(struct timer_list * timer);
+extern void add_timer(struct timer_lst * timer);
+extern int  del_timer(struct timer_lst * timer);
+extern void init_timer(struct timer_lst * timer);
 extern void adjust_timer(void);
 extern unsigned long timestamp(void);
